@@ -154,15 +154,16 @@
 /*	The directory with Postfix-writable data files (for example:
 /*	caches, pseudo-random numbers).
 /* .PP
-/*	Available in Postfix version 2.12 and later:
+/*	Available in Postfix version 3.0 and later:
 /* .IP "\fBmeta_directory (see 'postconf -d' output)\fR"
 /*	The location of non-executable files that are shared among
 /*	multiple Postfix instances, such as postfix-files, dynamicmaps.cf,
 /*	and the multi-instance template files main.cf.proto and master.cf.proto.
 /* .IP "\fBshlib_directory (see 'postconf -d' output)\fR"
-/*	The location of Postfix shared libraries (libpostfix-*.so),
-/*	and the default location of Postfix database plugins (libpostfix-*.so)
-/*	that have a relative pathname in the dynamicmaps.cf file.
+/*	The location of Postfix dynamically-linked libraries
+/*	(libpostfix-*.so), and the default location of Postfix database
+/*	plugins (postfix-*.so) that have a relative pathname in the
+/*	dynamicmaps.cf file.
 /* .PP
 /*	Other configuration parameters:
 /* .IP "\fBimport_environment (see 'postconf -d' output)\fR"
@@ -372,6 +373,7 @@
 #include <mail_conf.h>
 #include <mail_params.h>
 #include <mail_version.h>
+#include <mail_parm_split.h>
 
 /* Additional installation parameters. */
 
@@ -495,13 +497,27 @@ int     main(int argc, char **argv)
     mail_conf_read();
     get_mail_conf_str_table(str_table);
 
+    /*
+     * Alert the sysadmin that the backwards-compatible settings are still in
+     * effect.
+     */
+    if (var_compat_level < CUR_COMPAT_LEVEL) {
+	msg_info("Postfix is running with backwards-compatible default "
+		 "settings");
+	msg_info("See http://www.postfix.org/COMPATIBILITY_README.html "
+		 "for details");
+	msg_info("To disable backwards compatibility use \"postconf "
+		 VAR_COMPAT_LEVEL "=%d\" and \"postfix reload\"",
+		 CUR_COMPAT_LEVEL);
+    }
+
     /*
      * Environment import filter, to enforce consistent behavior whether this
      * command is started by hand, or at system boot time. This is necessary
      * because some shell scripts use environment settings to override
      * main.cf settings.
      */
-    import_env = argv_split(var_import_environ, ", \t\r\n");
+    import_env = mail_parm_split(VAR_IMPORT_ENVIRON, var_import_environ);
     clean_env(import_env->argv);
     argv_free(import_env);
 
@@ -544,7 +560,7 @@ int     main(int argc, char **argv)
      * Run the management script.
      */
     if (force_single_instance
-	|| argv_split(var_multi_conf_dirs, "\t\r\n, ")->argc == 0) {
+	|| argv_split(var_multi_conf_dirs, CHARS_COMMA_SP)->argc == 0) {
 	script = concatenate(var_daemon_dir, "/postfix-script", (char *) 0);
 	if (optind < 1)
 	    msg_panic("bad optind value");
@@ -560,7 +576,7 @@ int     main(int argc, char **argv)
 	if (*var_multi_wrapper == 0)
 	    msg_fatal("multi-instance support is requested, but %s is empty",
 		      VAR_MULTI_WRAPPER);
-	my_argv = argv_split(var_multi_wrapper, " \t\r\n");
+	my_argv = argv_split(var_multi_wrapper, CHARS_SPACE);
 	do {
 	    argv_add(my_argv, argv[optind], (char *) 0);
 	} while (argv[optind++] != 0);
